@@ -45,8 +45,8 @@ static char const RCSID[] =
 /* Are we running SUID or SGID? */
 int IsSetID = 0;
 
-static uid_t saved_uid = -2;
-static uid_t saved_gid = -2;
+static uid_t saved_uid = (uid_t) -2;
+static uid_t saved_gid = (uid_t) -2;
 
 /**********************************************************************
 *%FUNCTION: parsePacket
@@ -173,8 +173,8 @@ findTag(PPPoEPacket *packet, UINT16_t type, PPPoETag *tag)
 void
 switchToRealID (void) {
     if (IsSetID) {
-	if (saved_uid < 0) saved_uid = geteuid();
-	if (saved_gid < 0) saved_gid = getegid();
+	if (saved_uid == (uid_t) -2) saved_uid = geteuid();
+	if (saved_gid == (uid_t) -2) saved_gid = getegid();
 	if (setegid(getgid()) < 0) {
 	    printErr("setgid failed");
 	    exit(EXIT_FAILURE);
@@ -515,15 +515,16 @@ sendPADT(PPPoEConnection *conn, char const *msg)
     conn->session = 0;
 
     /* If we're using Host-Uniq, copy it over */
-    if (conn->useHostUniq) {
+    if (conn->hostUniq) {
 	PPPoETag hostUniq;
-	pid_t pid = getpid();
+	int len = (int) strlen(conn->hostUniq);
 	hostUniq.type = htons(TAG_HOST_UNIQ);
-	hostUniq.length = htons(sizeof(pid));
-	memcpy(hostUniq.payload, &pid, sizeof(pid));
-	memcpy(cursor, &hostUniq, sizeof(pid) + TAG_HDR_SIZE);
-	cursor += sizeof(pid) + TAG_HDR_SIZE;
-	plen += sizeof(pid) + TAG_HDR_SIZE;
+	hostUniq.length = htons(len);
+	memcpy(hostUniq.payload, conn->hostUniq, len);
+	CHECK_ROOM(cursor, packet.payload, len + TAG_HDR_SIZE);
+	memcpy(cursor, &hostUniq, len + TAG_HDR_SIZE);
+	cursor += len + TAG_HDR_SIZE;
+	plen += len + TAG_HDR_SIZE;
     }
 
     /* Copy error message */
